
package geometry;

public class Point {

	private final double x, y, z;

	public Point(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	public Point sumPoint(Point bod) {
		return new Point(x + bod.getX(), y + bod.getY(), z + bod.getZ());
	}

	public Point subPoint(Point bod) {
		return new Point(x - bod.getX(), y - bod.getY(), z - bod.getZ());
	}

	public Point mulPoint(Point bod) {
		double a = y * bod.getZ() - z * bod.getY();
		double b = z * bod.getX() - x * bod.getZ();
		double c = x * bod.getY() - y * bod.getX();
		return new Point(a, b, c);
	}

	public double mulSkal(Point b) {
		return x * b.getX() + y * b.getY() + z * b.getZ();
	}

	public Point mulConst(double c) {
		return new Point(x * c, y * c, z * c);
	}

	public double size() {
		return Math.sqrt(Math.abs(x * x + y * y + z * z));
	}

	public Point setNorm() {
		double s = size();
		return new Point(x / s, y / s, z / s);
	}

	public Point setNormToPoint(Point a) {
		Point p = subPoint(a);
		double s = p.size();

		return new Point(p.getX() / s, p.getY() / s, p.getZ() / s);
	}

	public double cosTheta(Point b) {
		double x = mulSkal(b);
		double y = size() * b.size();
		return x / y;
	}

	public double distance(Point a) {
		return Math.sqrt(
				(x - a.getX()) * (x - a.getX()) + (y - a.getY()) * (y - a.getY()) + (z - a.getZ()) * (z - a.getZ()));
	}

	public Point reflected(Point n) {

		double r = this.mulSkal(n);
		Point rr = n.mulConst(r);
		rr = rr.mulConst(-2);
		Point reflect = this.sumPoint(rr).mulConst(-1);
		return reflect;
	}
	

	public double getX() {
		return x;
	}

	public double getY() {
		return y;
	}

	public double getZ() {
		return z;
	}

}