package graphics;

import geometry.Matrix4;
import geometry.Point;
import geometry.Vector;
import objects.Plane;

public class Camera {

	private Point camPos;
	private Point camL;
	private Point e;

	private Plane proj;

	private int rWidth;
	private int rHeight;
	private int pxX;
	private int pxY;
	private double f;
	private Matrix4 m;

	private Point u = new Point(1, 0, 0);
	private Point v = new Point(0, 0, 1);

	public Camera(Point camPos, Point camL, double f, int[] screen) {
		this.camPos = camPos;
		this.camL = camL;
		this.f = f;
		this.pxX = screen[0];
		this.pxY = screen[1];
		this.rWidth = screen[2];
		this.rHeight = screen[3];
	}

	public void setParams(int x, int axis) {
		e = camPos.setNormToPoint(camL);
		e = e.mulConst(10).sumPoint(camPos);
		transM(Math.toRadians(x), axis);
	}

	public Vector getRay(double x, double y) {
		double a = (x * rWidth / pxX) - (rWidth * 0.5);
		double b = (-y * rHeight / pxY) + (rHeight * 0.5);
		Point u1 = proj.getL().mulConst(a);
		u1 = u1.sumPoint(proj.getK());
		Point v1 = proj.getM().mulConst(b);
		Point pr = v1.sumPoint(u1);
		Point dir = pr.setNormToPoint(e);
		Vector ray = new Vector(this.e, dir);
		return ray;
	}

	private void transM(double rad, int axis) {
		m = new Matrix4(rad, axis);
		camPos = m.mul(camPos);
		e = m.mul(e);
		u = m.mul(u);
		v = m.mul(v);
		proj = new Plane(camPos, u, v);
	}

	public Point getCamPos() {
		return camPos;
	}

	public void setCamPos(Point camPos) {
		this.camPos = camPos;
	}

	public Point getCamL() {
		return camL;
	}

	public void setCamL(Point camL) {
		this.camL = camL;
	}

	public double getF() {
		return f;
	}

	public void setF(double f) {
		this.f = f;
	}

	public int getPxX() {
		return pxX;
	}

	public int getPxY() {
		return pxY;
	}

}
