package graphics;

import java.awt.Color;

import geometry.Point;
import geometry.Intersection;
import geometry.Vector;
import objects.ObjectsList;
import objects.Objects;

public class GPU {
	private long rays = 0;
	private ObjectsList objectsList;
	private int depthRec = 10;
	private Camera cam;
	Point lightPos = new Point(0, 0, 2.3);

	public GPU() {

	}

	public int[] rayTrace(Vector p, int depth) {
		int[] color = { 255, 255, 255 };
		rays++;
		Intersection in = objectsList.findCloser(p);
		if (in == null) {
			return color;
		} else {

			Vector rayV = new Vector(in.getIntersection(), p.getDir().mulConst(-1));

			color = countLightning(in, rayV);
			depth += 1;
			Objects t = in.getT();
			Point n = in.getNormal();

			if (depth <= depthRec) {

				if (t.getSurface().getPhongModel().getrS() > 0) {
					color = countReflect(t, n, rayV, color, depth);
				}

				if (t.getSurface().getLom()[0] >= 1) {
					color = countRefract(t, n, rayV, color, depth);
				}
			}
			return color;
		}
	}

	public Color drawPix(int x, int y) {
		int[] color0 = null;
		color0 = rayTrace(cam.getRay(x + 0.5, y + 0.5), 0);
		if (color0[0] > 255)
			color0[0] = 255;
		if (color0[1] > 255)
			color0[1] = 255;
		if (color0[2] > 255)
			color0[2] = 255;
		if (color0[0] < 0)
			color0[0] = 0;
		if (color0[1] < 0)
			color0[1] = 0;
		if (color0[2] < 0)
			color0[2] = 0;
		Color color = new Color((int) color0[0], (int) color0[1], (int) color0[2]);
		return color;
	}


	private int[] countLightning(Intersection inter, Vector ray) {

		Objects o = inter.getT();
		Point normal = inter.getNormal();
		int[] colorA = { 0, 0, 0 };
		int[] color = { 0, 0, 0 };

		Point pLight = lightPos;
		Point dirLight = pLight.setNormToPoint(inter.getIntersection());
		Vector shadow = new Vector(inter.getIntersection(), dirLight);
		Point reflLight = dirLight.reflected(normal);

		float shade = objectsList.shadowRay(shadow, pLight);
		if (shade > 0) {

			float koef = o.getSurface().getPhongModel().countKoef(ray.getDir(), dirLight, normal, reflLight.setNorm());

			Color colorL = o.getSurface().getColorUV(inter.getUv());

			color[0] = (int) (koef * colorL.getRed() * shade);
			color[1] = (int) (koef * colorL.getGreen() * shade);
			color[2] = (int) (koef * colorL.getBlue() * shade);

		} else {
			color[0] = 0;
			color[1] = 0;
			color[2] = 0;
		}
		colorA[0] += color[0];
		colorA[1] += color[1];
		colorA[2] += color[2];

		return colorA;
	}

	private int[] countRefract(Objects t, Point n, Vector rayV, int[] color, int depth) {
		float[] refract = t.getSurface().getLom();
		Vector refracted = rayV.refractedT(refract[0], refract[1], n);
		int[] refColor = rayTrace(refracted, depth);
		refColor = mulField(refColor, t.getSurface().getColorFloat());
		return sumField(refColor, color, 1);
	}

	private int[] countReflect(Objects t, Point n, Vector rayV, int[] color, int depth) {
		Vector reflect = new Vector(rayV.getPos(), rayV.getDir().reflected(n));
		int[] reflectColor = rayTrace(reflect, depth);
		reflectColor = mulField(reflectColor, t.getSurface().getColorFloat());
		return color = sumField(reflectColor, color, 0.06);
	}

	private int[] sumField(int[] b1, int[] b2, double k) {
		int[] res = { (int) (b1[0] * k) + b2[0], (int) (b1[1] * k) + b2[1], (int) (b1[2] * k) + b2[2] };
		return res;
	}

	private int[] mulField(int[] i, float[] f) {
		int[] v = new int[3];
		v[0] = (int) (i[0] * f[0]);
		v[1] = (int) (i[1] * f[1]);
		v[2] = (int) (i[2] * f[2]);
		return v;
	}

	public long getRays() {
		return rays;
	}

	public void setRays(long rays) {
		this.rays = rays;
	}

	public void setCamera(Camera camera) {
		this.cam = camera;
	}

	public int getDepthRec() {
		return depthRec;
	}

	public void setDepthRec(int depthRec) {
		this.depthRec = depthRec;
	}

	public void setObjectsList(ObjectsList ObjectsList) {
		this.objectsList = ObjectsList;
	}

}