package gui;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;

import geometry.Point;
import graphics.Camera;
import graphics.PhongModel;
import objects.Square;
import objects.Sphere;
import objects.Plane;
import objects.ObjectsList;
import objects.Objects;
import objects.Cylinder;
import objects.props.Surface;
import objects.props.Texture;

public class Scene {
	ObjectsList ObjectsList;
	Camera cam;

	public Scene() {

		Point camPos = new Point(0, 5, 0);
		Point camL = new Point(0, 0, 0);
		double f = 10;
		int[] canvas = { 800, 800, 4, 4 };
		cam = new Camera(camPos, camL, f, canvas);
		cam.setParams(0, 0);

		ObjectsList = new ObjectsList();
		BufferedImage texFloor = null;
		BufferedImage texFim = null;

		try {
			texFloor = ImageIO.read(getClass().getResource("/pic/ground.png"));
			texFim = ImageIO.read(getClass().getResource("/pic/fim.jpg"));
		} catch (IOException e) {
			e.printStackTrace();
		}

		Texture sachTex = new Texture(texFloor);
		Texture uhkTex = new Texture(texFim);

		float[] rGlass = { (float) 1.52, 1, (float) 0.025 };
		float[] rInnerGlass = { 1, (float) 1.52, (float) 0.025 };
		float[] rWall = { 0, 1, 0 };

		PhongModel transparentPhong = new PhongModel(0.2f, 0.6f, 0, 20);
		PhongModel wallPhong = new PhongModel(0.8f, 0, 0.2f, 1);

		Surface yellow = new Surface(Color.YELLOW, wallPhong, rWall);
		Surface blue = new Surface(Color.BLUE, wallPhong, rWall);
		Surface red = new Surface(Color.RED, wallPhong, rWall);
		Surface floor = new Surface(Color.WHITE, wallPhong, rWall, sachTex);
		Surface wall = new Surface(Color.ORANGE, wallPhong, rWall, uhkTex);
		Surface glass = new Surface(Color.WHITE, transparentPhong, rGlass);
		Surface glassI = new Surface(Color.WHITE, transparentPhong, rInnerGlass);

		Point centerBall1 = new Point(0, 0, 0);
		Objects obj1 = new Sphere(centerBall1, 0.8, glass);
		ObjectsList.add(obj1);

		Point centerBall2 = new Point(-1.2, 1.2, -1.4);
		Objects obj2 = new Sphere(centerBall2, 0.9, glass);
		ObjectsList.add(obj2);

		Point centerCyl = new Point(1.5, 1, 0);
		Objects obj3 = new Cylinder(centerCyl, 0.5, glass, 2.5);
		ObjectsList.add(obj3);

		Point centerBall3 = new Point(-1.5, 2, 1.5);
		Objects obj5 = new Sphere(centerBall3, 0.6, glass);
		ObjectsList.add(obj5);

		Objects obj6 = new Sphere(centerBall3, 0.55, glassI);
		ObjectsList.add(obj6);

		Point i1 = new Point(0, 0, -2.5);
		Point j1 = new Point(1, 0, 0);
		Point k1 = new Point(0, 1, 0);

		Objects plane1 = new Plane(i1, j1, k1);
		Objects square1 = new Square((Plane) plane1, 2.5, floor);
		ObjectsList.add(square1);

		Point i2 = new Point(0, -2.5, 0);
		Point j2 = new Point(1, 0, 0);
		Point k2 = new Point(0, 0, 1);

		Objects plane2 = new Plane(i2, j2, k2);
		Objects square2 = new Square((Plane) plane2, 2.5, wall);
		ObjectsList.add(square2);

		Point i3 = new Point(-2.5, 0, 0);
		Point j3 = new Point(0, 1, 0);
		Point k3 = new Point(0, 0, 1);

		Objects plane3 = new Plane(i3, j3, k3);
		Objects square3 = new Square((Plane) plane3, 2.5, yellow);
		ObjectsList.add(square3);

		Point i4 = new Point(2.5, 0, 0);
		Point j4 = new Point(0, 1, 0);
		Point k4 = new Point(0, 0, 1);

		Objects plane4 = new Plane(i4, j4, k4);
		Objects square4 = new Square((Plane) plane4, 2.5, blue);
		ObjectsList.add(square4);

		Point i5 = new Point(0, 0, 2.5);
		Point j5 = new Point(1, 0, 0);
		Point k5 = new Point(0, 1, 0);

		Objects plane5 = new Plane(i5, j5, k5);
		Objects square5 = new Square((Plane) plane5, 2.5, red);
		ObjectsList.add(square5);
	}

	public ObjectsList getObjectsList() {
		return ObjectsList;
	}

	public Camera getCamera() {
		return cam;
	}
}