package gui;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.image.MemoryImageSource;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import graphics.GPU;

public class Window extends JFrame {

	private static final long serialVersionUID = 1L;
	private int widthC;
	private int heightC;
	private WindowCanvas canvas;
	private int[] pixel;
	private Image pict;

	private JPanel panel;
	private MemoryImageSource image;
	private JLabel raysNote;
	private Scene data;
	private GPU gpu;
	private Action actRayTrace;
	private Action actRayCast;
	private JButton btnRayTrace;
	private JButton btnRayCast;

	public Window(int width, int height) {
		super("Transparent Scenes - Ray-Tracing");

		init();

		this.heightC = height;
		this.widthC = width;
		pixel = new int[height * width];
		image = new MemoryImageSource(width, height, pixel, 0, width);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		actRayTrace = new AbstractAction() {

			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {

				gpu.setDepthRec(5);
				gpu.setRays(0);

				countField();
				raysNote.setText(String.valueOf(gpu.getRays()));

			}

		};
		actRayCast = new AbstractAction() {

			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent arg0) {
				gpu.setDepthRec(0);
				gpu.setRays(0);

				countField();
				raysNote.setText(String.valueOf(gpu.getRays()));
			}
		};
		canvas = new WindowCanvas(width, height);
		panel = new JPanel();
		btnRayTrace = new JButton("Ray-Tracing");
		btnRayTrace.addActionListener(actRayTrace);
		panel.add(btnRayTrace);

		btnRayCast = new JButton("Ray-Casting");
		btnRayCast.addActionListener(actRayCast);
		panel.add(btnRayCast);

		JLabel rays = new JLabel("Počet paprsků: ");
		panel.add(rays);

		raysNote = new JLabel(" 0");
		panel.add(raysNote);

		getContentPane().add(panel, BorderLayout.SOUTH);
		getContentPane().add(canvas, BorderLayout.CENTER);
		setVisible(true);
		pack();
	}

	public void setPixel(int pos, int color) {
		pixel[pos] = color;
	}

	public void drawPic() {
		image.newPixels();
		pict = createImage(image);
		canvas.drawPic(pict);
	}

	private void init() {
		gpu = new GPU();
		data = new Scene();
		gpu.setObjectsList(data.getObjectsList());

		gpu.setCamera(data.getCamera());
	}

	private void countField() {
		for (int k = 0; k < getHeightC(); k++) {
			for (int l = 0; l < getWidthC(); l++) {
				int color = gpu.drawPix(l, k).getRGB();
				setPixel(l + k * getWidthC(), color);
			}
			drawPic();
		}
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {

				new Window(800, 800).setVisible(true);
			}
		});
	}
	
	public int getWidthC() {
		return widthC;
	}

	public int getHeightC() {
		return heightC;
	}
}