
package objects;

import geometry.Point;
import geometry.Intersection;
import geometry.Vector;
import objects.props.Surface;

public class Cylinder extends Objects {
	private Point center;
	private double radius;
	private double height;

	public Cylinder(Point center, double radius, Surface surface, double height) {
		this.center = center;
		this.radius = radius;
		this.surface = surface;
		this.height = height;
	}

	@Override
	public Intersection findIntersection(Vector p) {
		Point pr;
		double[] t = getParameterT(p);
		if (t == null) {
			return null;
		} else {
			if (t[0] < t[1] && t[0] > 0.1e-10) {
				pr = p.getPos().sumPoint(p.getDir().mulConst(t[0]));
				if (pr.getZ() < height && pr.getZ() > -height) {
					return new Intersection(this, pr, getNormal(pr), t[0], null);
				}
			} else if (t[0] < t[1] && t[1] > 0.1e-10) {
				pr = p.getPos().sumPoint(p.getDir().mulConst(t[1]));
				if (pr.getZ() < height && pr.getZ() > -height) {
					return new Intersection(this, pr, getNormal(pr), t[1], null);
				}
			}
		}
		return null;
	}

	private double[] getParameterT(Vector p) {
		double x1 = p.getPos().getX();
		double y1 = p.getPos().getY();
		double i = p.getDir().getX();
		double j = p.getDir().getY();
		double l = center.getX();
		double m = center.getY();
		double a = i * i + j * j;
		double b = 2 * i * (x1 - l) + 2 * j * (y1 - m);
		double c = (x1 - l) * (x1 - l) + (y1 - m) * (y1 - m) - radius * radius;
		return getQuadricEquation(a, b, c);
	}

	private Point getNormal(Point intersect) {
		Point s = intersect.subPoint(center);
		Point n = new Point(s.getX(), s.getY(), 0);
		return n.setNorm();
	}

	private double[] getQuadricEquation(double a, double b, double c) {

		double[] t = new double[2];
		double d = (b * b) - (4 * a * c);
		if (d == 0) {
			double t1 = -b / (2 * a);
			t[0] = t1;
			t[1] = t1;
			return t;
		}
		if (d > 0.5e-10) {
			double t1 = (-b + Math.sqrt(d)) / (2 * a);
			double t0 = (-b - Math.sqrt(d)) / (2 * a);
			t[0] = t0;
			t[1] = t1;
			return t;
		} else {
			return null;
		}
	}
}