
package objects;

import geometry.Point;
import geometry.Intersection;
import geometry.Vector;
import objects.props.Surface;

public class Plane extends Objects {
	private Point k;
	private Point l;
	private Point m;
	private Point normal;

	public Plane() {

	}

	public Plane(Point k, Point l, Point m) {
		this.k = k;
		this.l = l;
		this.m = m;
		normal = normal();
	}

	public Plane(Point k, Point l, Point m, Surface surface) {
		this.k = k;
		this.l = l;
		this.m = m;
		this.surface = surface;
		normal = normal();
	}

	public Intersection findIntersection(Vector v) {
		double d = -k.mulSkal(normal);
		double v0 = -(normal.mulSkal(v.getPos()) + d);
		double vd = normal.mulSkal(v.getDir());
		Point normal = this.normal;
		if (vd > 0) {
			normal = normal.mulConst(-1);
		}
		if (vd == 0) {
			return null;
		}
		double t = v0 / vd;

		if (t < 0.5e-10) {
			return null;
		} else {
			return new Intersection(this, v.getPos().sumPoint(
					v.getDir().mulConst(t)), normal, t, null);
		}
	}

	private Point normal() {
		Point u = l.setNorm();
		Point v = m.setNorm();
		Point n = u.mulPoint(v);

		return n.setNorm();
	}

	public Point getK() {
		return k;
	}

	public void setK(Point k) {
		this.k = k;
	}

	public Point getL() {
		return l;
	}

	public void setL(Point l) {
		this.l = l;
	}

	public Point getM() {
		return m;
	}

	public void setM(Point m) {
		this.m = m;
	}

	public Point getNormal() {
		return normal;
	}

}