
package objects;

import geometry.Point;
import geometry.Intersection;
import geometry.Vector;
import objects.props.Surface;

public class Sphere extends Objects {
	private Point center;
	private double radius;

	public Sphere(Point center, double radius, Surface surface) {
		this.center = center;
		this.radius = radius;
		this.surface = surface;
	}

	public Intersection findIntersection(Vector p) {
		double t = getParameterT(p);
		if (t < 0) {
			return null;
		} else {
			Point intersection = p.mulConst(t);
			Point normal = getNormal(intersection);
			return new Intersection(this, intersection, normal, t, null);
		}

	}

	private double getParameterT(Vector p) {
		double i = p.getDir().getX();
		double j = p.getDir().getY();
		double k = p.getDir().getZ();
		double l = center.getX();
		double m = center.getY();
		double n = center.getZ();
		double x1 = p.getPos().getX();
		double y1 = p.getPos().getY();
		double z1 = p.getPos().getZ();

		double a = i * i + j * j + k * k;
		double b = 2 * i * (x1 - l) + 2 * j * (y1 - m) + 2 * k * (z1 - n);
		double c = p.getPos().subPoint(center).mulSkal(p.getPos().subPoint(center)) - radius * radius;

		return getQuadricEquation(a, b, c);
	}

	private Point getNormal(Point pt) {
		double x = (pt.getX() - center.getX()) / radius;
		double y = (pt.getY() - center.getY()) / radius;
		double z = (pt.getZ() - center.getZ()) / radius;
		return new Point(x, y, z);
	}

	private double getQuadricEquation(double a, double b, double c) {
		double d = (b * b) - (4 * a * c);
		double t0, t1;
		if (d >= 0 && d < 0.5e-10) {
			double t = (-b) / (2 * a);
			return t;
		}
		if (d > 0.5e-10) {
			t1 = (-b + Math.sqrt(d)) / (2 * a);
			if (t1 < 0.5e-10) {
				return -1;
			} else {
				t0 = (-b - Math.sqrt(d)) / (2 * a);
				if (t0 > 0.5e-10) {
					return t0;
				} else {
					return t1;
				}
			}
		} else {
			return -1;
		}
	}

}