
package objects;

import geometry.Point;
import geometry.Intersection;
import geometry.Vector;
import objects.props.Surface;

public class Square extends Objects {
	private double size;
	private Plane r;
	private Point s1;
	private Point s2;

	public Square(Plane r, double size) {
		this.r = r;
		this.size = size;
		Point k = r.getK();
		Point l = r.getL().setNorm().mulConst(size);
		Point m = r.getM().setNorm().mulConst(size);

		Point l0 = r.getL().setNorm().mulConst(size * -1);
		Point m0 = r.getM().setNorm().mulConst(size * -1);
		this.s1 = l.sumPoint(k.sumPoint(m));
		this.s2 = k.sumPoint(l0.sumPoint(m0));
	}

	public Square(Plane r, double size, Surface surface) {
		this.r = r;
		this.surface = surface;
		this.size = size;
		Point l = r.getL().setNorm().mulConst(size);
		Point m = r.getM().setNorm().mulConst(size);
		Point k = r.getK();
		Point l0 = r.getL().setNorm().mulConst(size * -1);
		Point m0 = r.getM().setNorm().mulConst(size * -1);
		this.s1 = l.sumPoint(k.sumPoint(m));
		this.s2 = k.sumPoint(l0.sumPoint(m0));
	}

	@Override
	public Intersection findIntersection(Vector v) {

		Intersection p = (Intersection) r.findIntersection(v);

		if (p == null) {
			return null;
		} else {
			double x = p.getIntersection().getX();
			double y = p.getIntersection().getY();
			double z = p.getIntersection().getZ();

			if (x <= s1.getX() + 0.1e-10 && x >= s2.getX() - 0.1e-10 && y <= s1.getY() + 0.1e-10
					&& y >= s2.getY() - 0.1e-10 && z <= s1.getZ() + 0.1e-10 && z >= s2.getZ() - 0.1e-10) {
				double u0 = x * r.getL().getX() + y * r.getL().getY() + z * r.getL().getZ();
				double v0 = x * r.getM().getX() + y * r.getM().getY() + z * r.getM().getZ();
				double[] uv = new double[2];
				uv[0] = (u0 + size) / (size * 2);

				if (u0 >= 0 && v0 >= 0 || u0 < 0 && v0 > 0) {
					uv[1] = Math.abs(v0 - size) / (size * 2);
				} else {
					uv[1] = (Math.abs(v0) + size) / (size * 2);
				}

				return new Intersection(this, p.getIntersection(), p.getNormal(), p.getParameterT(), uv);

			} else {
				return null;
			}
		}
	}

	public double getSize() {
		return size;
	}

	public Plane getR() {
		return r;
	}

}
